/* 
	Author 	: Thomas GIL (DotNetGuru.org)
	Date 	: 17 september 2006
	License : Public Domain
*/
using Base;
using DotNetGuru.AspectDNG.Joinpoints;

namespace Aspects {
    public class TestAroundCall {
        const string Target = "//Type[@FullName='Base.TestAroundCall']/Method/Call";

        [AroundCall(Target + "[contains(@Operand, '::RealTarget(') or contains(@Operand, '::DoNothingTarget(')]")]
        public static object InterceptorPrimitiveTypes(MethodJoinPoint jp) {
            int val1 = (int)jp[0];
            double val2 = (double)jp[1];
            return val1 + (int)val2;
        }

        [AroundCall(Target + "[contains(@Operand, '::RealTargetMixed(')]")]
        public static object InterceptorMixedTypes(MethodJoinPoint jp) {
            string s = (string)jp[2];
            int i = (int)jp.Proceed();
            return s.Length + i;
        }

        [AroundCall(Target + "[contains(@Operand, '::StaticRealTargetMixed(')]")]
        public static object StaticInterceptorMixedTypes(MethodJoinPoint jp) {
            string s = (string)jp[2];
            int i = (int)jp.Proceed();
            return s.Length + i;
        }

        [AroundCall("* Base.A::.ctor(System.String)")]
        public static object AroundConstructor(ConstructorJoinPoint jp) {
            jp[0] = "intercepted " + jp[0];
            return jp.Proceed();
        }

        [AroundCall("//Instruction[match(@Operand, '* System.Console::Write*(System.String)')][not(match(ancestor::Type[1], 'Aspects.*'))]")]
        public static object AroundExternalCall(MethodJoinPoint jp) {
            System.Console.Write(""); // To check whether this invocation is excluded (else -> stack overflow)
            return jp.Proceed();
        }
    }
}