/* 
	Author 	: Thomas GIL (DotNetGuru.org)
	Date 	: 17 september 2006
	License : Public Domain
*/
using System;
using System.Diagnostics;
using DotNetGuru.UnitTestFramework;

using System.Data.SqlClient;

namespace Base {
	public class TypesOfTypes : TestCase {
	
		public class BaseType {
			public virtual int Compute(int i){
				return i *2;
			}
		}
		public class ChildType : BaseType{
			public override int Compute(int i){
				return base.Compute(i) *2;
			}
		}
	
		public enum Values : uint{
			One, Two
		}
		
		public class MyException : Exception{
			private static int i = 5;
		}
		
		public struct StructType{
			public static int m_StaticValue;
			public static int StaticValue {
				get { return m_StaticValue; }
				set { m_StaticValue = value; }
			}
			
			public int m_value;
			public int Value {
				get { return m_value; }
				set { m_value = value; }
			}
			
			public StructType(int i){ m_value = i;  }
			
			public void Increment() { ++ m_value; }
			public static void StaticIncrement() { ++ m_StaticValue; }
			
			public override string ToString (){
				return "Struct type : " + m_value;
			}
		}
	
		public delegate StructType[] D(StructType[] s);
		public event D d;

		public void Test(){
			StructType s2 = new StructType(5);
			s2.Increment();
			AssertEqual(new StackFrame(), 6, s2.Value);

			StructType.StaticValue = 5;
			StructType.StaticIncrement();
			AssertEqual(new StackFrame(), 6, StructType.StaticValue);
			
			d = new D(DMethod);
			d+=new D(DMethod);
			d-=new D(DMethod);
			d = ReturnDMethod(d);
			s2 = d(d(new StructType[]{s2}))[0];
			AssertEqual(new StackFrame(), 6, StructType.StaticValue);
			
			uint i = this;
			AssertEqual(new StackFrame(), 1u, i);
			
			i = this + 2u;
			AssertEqual(new StackFrame(), 3u, i);
			
			try{ throw new MyException();}
			catch{}
			
			int[] results = DoubleValues(new int[]{1, 2, 3});
			AssertEqual(new StackFrame(), 4, results[1]);
			
			results = Sums(new int[,]{{1,2}, {3,4}});
			AssertEqual(new StackFrame(), 7, results[1]);
			
			int[,] otherResults = DoubleValues(new int[,]{{1,2}, {3,4}});
			AssertEqual(new StackFrame(), 4, otherResults[0, 1]);
			
			AssertEqual(new StackFrame(), Values.One, Echo (Values.One));
			
			int first = 3, second = 4;
			Swap(ref first, ref second);
			AssertEqual(new StackFrame(), 3, second);
			
			BaseType bt = new ChildType();
			AssertEqual(new StackFrame(), 4, bt.Compute(1));
			
			AssertEqual(new StackFrame(), 4, LockingMethod());
			
			try{ 
				LockingThrowingMethod(); 
				AssertTrue(new StackFrame(), false);
			}
			catch{}
		}
		
		public Values Echo(Values data){
			return data;
		}
		
		public void Swap(ref int i, ref int j){
			int tmp = i;
			i = j;
			j = tmp;
		}

		public int[] DoubleValues(int[] data){
			int[] result = new int[data.Length];
			for (int i=0; i<data.Length; i++) {
				result[i] = data[i] * 2;
			}
			return result;
		}
		
		public int[] Sums(int[,] data){
			int[] result = new int[data.GetLength(0)];
			for (int i=0; i<data.GetLength(0); i++) {
				for (int j=0; j<data.GetLength(1); j++) {
					result[i] += data[i,j];
				}
			}
			return result;
		}
		
		public int[,] DoubleValues(int[,] data){
			for (int i=0; i<data.GetLength(0); i++) {
				for (int j=0; j<data.GetLength(1); j++) {
					data[i,j] *= 2;
				}
			}
			return data;
		}
		
		public StructType[] DMethod(StructType[] s){
			return s;
		}
		
		public D ReturnDMethod(D d){
			return d;
		}
		
		public static implicit operator uint(TypesOfTypes tt){
			return 1;
		}
		public static int operator+(TypesOfTypes tt, int i){
			return i+1;
		}
		
		public int LockingMethod(){
			lock(this){
				return 4;
			}
		}
		public int LockingThrowingMethod(){
			lock(this){
				throw new Exception("normal");
			}
		}
	}
}

