/* 
	Author 	: Thomas GIL (DotNetGuru.org)
	Date 	: 17 september 2006
	License : Public Domain
*/
#if MONO_ASSEMBLY
using System;
using System.Diagnostics;
using System.IO;
using System.Xml.XPath;
using System.Reflection;
using DotNetGuru.UnitTestFramework;
using DotNetGuru.AspectDNG.XPath;

namespace XmlTests{
	public class Person{
		private string m_Name;
		public string Name{
			get{return m_Name;}
			set{m_Name = value;}
		}
		
		private string m_FirstName;
		public string FirstName{
			get{return m_FirstName;}
			set{m_FirstName = value;}
		}
		private int m_Age;
		public int Age{
			get{return m_Age;}
			set{m_Age = value;}
		}
		
		public Person(string name,string firstname,int age){
			Name = name;
			FirstName = firstname;
			Age = age;
		}
	}
	
	public class XPathNavigator : TestCase {
		public void Test(){
			Person p = new Person("Gil","Tom",29);
			Navigator root = new Navigator(p);
			
			Navigator nav = (Navigator) root.Clone();
			AssertEqual(XPathNodeType.Root, nav.NodeType);
			
			AssertFalse(nav.MoveToNext()); // Only one root
			AssertEqual(XPathNodeType.Root, nav.NodeType);
			
			AssertTrue(nav.MoveToFirstChild()); // Now points to a Person
			AssertFalse(nav.MoveToFirstChild()); // Person has no child
			AssertEqual(XPathNodeType.Element, nav.NodeType);
			AssertEqual("Person", nav.LocalName);
			AssertEqual(typeof(Person).FullName, nav.Value);
			AssertSame(p, nav.RealObject);

			AssertEqual("Gil", nav.GetAttribute("Name", string.Empty));
			
			AssertTrue(nav.MoveToAttribute("Name",string.Empty)); // Now points to the Name property
			AssertEqual(XPathNodeType.Attribute, nav.NodeType);			
			AssertEqual("Name", nav.LocalName);
			AssertEqual("Gil", nav.Value);
			
			AssertTrue(nav.MoveToParent()); 
			AssertEqual(XPathNodeType.Element, nav.NodeType);
			AssertTrue(nav.MoveToFirstAttribute()); 
			AssertEqual("Age", nav.LocalName);
			AssertEqual("29", nav.Value);
			
			AssertTrue(nav.MoveToNextAttribute());
			AssertEqual("FirstName", nav.LocalName);
			AssertEqual("Tom", nav.Value);
		}
	}
}
#endif