/* 
	Author 	: Thomas GIL (DotNetGuru.org)
	Date 	: 17 september 2006
	License : Public Domain
*/
using System;
using System.Diagnostics;

namespace DotNetGuru.UnitTestFramework{
    public class TestCase {
		public static int NbFailures;

        private void Print(StackFrame frame, string message, params object[] data) {
            ++NbFailures;

            Console.Error.WriteLine("[{0}.{1}] ",
                frame.GetMethod().DeclaringType.FullName,
                frame.GetMethod().Name);

            Console.Error.WriteLine("\t" + message, data);
        }

        public static void ExpectedException(StackFrame frame, Type exceptionType) {
			++NbFailures;
			Console.Error.WriteLine("[ExpectedException] " + exceptionType);
		}

        protected void AssertTrue(StackFrame frame, bool condition) {
            if (!condition) { Print(frame, "Expected true", ""); }
		}

        protected void AssertFalse(StackFrame frame, bool condition) {
            if (condition) { Print(frame, "Expected false", ""); }
		}

        protected void AssertNull(StackFrame frame, object obj) {
            if (obj != null) { Print(frame, "Expected null, got {0}", obj); }
		}

        protected void AssertNotNull(StackFrame frame, object obj) {
            if (obj == null) { Print(frame, "Expected not null", ""); }
		}

        protected void AssertFail(StackFrame frame, string message) {
            Print(frame, message, "");
		}

        protected void AssertEqual(StackFrame frame, object expected, object value) {
            if (!expected.Equals(value)) { Print(frame, "Expected {0}, Got {1}", expected, value); }
        }

        protected void AssertNotEqual(StackFrame frame, object expected, object value) {
            if (expected.Equals(value)) { Print(frame, "Expected not {0}, Got {1}", expected, value); }
		}

        protected void AssertSame(StackFrame frame, object expected, object value) {
            if (!Object.ReferenceEquals(expected, value)) { Print(frame, "Expected {0}, Got {1}", expected, value); }
		}

        protected void AssertNotSame(StackFrame frame, object expected, object value) {
            if (Object.ReferenceEquals(expected, value)) { Print(frame, "Expected not {0}, Got {1}", expected, value); }
		}

        protected void AssertLessThan(StackFrame frame, double expected, double value) {
            if (expected <= value) { Print(frame, "Expected maximum {0}, Got {1}", expected, value); }
		}

        protected void AssertGreaterThan(StackFrame frame, double expected, double value) {
            if (expected >= value) { Print(frame, "Expected minimum {0}, Got {1}", expected, value); }
		}
    }
}